/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2005 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JonasAdminTestCase.java,v 1.9 2005/07/25 12:29:58 kemlerp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jonasadmin.test.util;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import junit.framework.TestCase;

import org.objectweb.jonas.adm.AdmInterface;

import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebResponse;

/**
 * Define a class to add useful methods for test the examples
 *  - Deploy ear, war and beans
 *  - Retrieve initial context
 * @author Florent Benoit
 */
public class JonasAdminTestCase extends TestCase {

    /**
     * URL of the login page
     */
    protected static final String URL_JONASADMIN = "/jonasAdmin/";

    /**
     * NAME of FRAME content
     */
    protected static final String FRAME_CONTENT = "content";

    /**
     * NAME of FRAME tree
     */
    protected static final String FRAME_TREE = "tree";

    /**
     * NAME of FRAME top
     */
    protected static final String FRAME_TOP = "top";

    /**
     * URL of the deployment of EARs
     */
    protected static final String URL_JONASADMIN_DEPLOYEAR = "/jonasAdmin/EditDeploy.do?type=ear";

    /**
     * URL of the deployment of JARs
     */
    protected static final String URL_JONASADMIN_DEPLOYJAR = "/jonasAdmin/EditDeploy.do?type=jar";

    /**
     * Name of the property file of jonasAdmin tests
     */
    private static final String JONASADMINTEST_PROPERTY_NAME = "jonasAdminTests.properties";

    /**
     * Name of the JOnAS server used for tests
     */
    private static String jonasName = "jonas";

    /**
     * Initial context used for lookup
     */
    private static Context ictx = null;

    /**
     * JOnAS admin used for communicate via JMX to JOnAS
     */
    private static AdmInterface admI = null;

    /**
     * Conversation used for HttpUnit
     */
    protected WebConversation wc = null;

    /**
     * URL used for the constructor
     */
    protected String url = null;

    /**
     * URL used to return to Welcome.jsp
     */
    protected String urlWelcome = null;

    /**
     * URL used to log out
     */
    protected String urlLogOut = null;


    /**
     * Prefix for build URLs
     */
    protected String prefixUrl = null;

    /**
     * JOnAS and jonasAdmin properties
     */
    protected JProperties jProp = null;

    /**
     * jonasAdmin test properties
     */
    protected Properties configFile;

    /**
     * jonasAdmin port
     */
    protected String port;

    /**
     * Add to the specified url the prefix
     * @param url relative URL
     * @return absolute path of URL
     */
    protected String getAbsoluteUrl (String url) {
        return (this.prefixUrl + url);
    }

    /**
     * Initialize the port used by tests and the prefix
     */
    private void init() {
        port = System.getProperty("http.port");
        if (port == null) {
            port = "9000";
        }

        prefixUrl = "http://localhost:" + port;

        urlWelcome = prefixUrl + "/jonasAdmin/Welcome.do";
        urlLogOut = prefixUrl + "/jonasAdmin/logOut.do";

        jProp = new JProperties();

        try {
            configFile = jProp.getProperties(JONASADMINTEST_PROPERTY_NAME);
        } catch (Exception e) {
            System.err.println("Cannot find file : " + JONASADMINTEST_PROPERTY_NAME + ". " + e);
            e.printStackTrace();
        }
    }

    /**
     * Constructor with a specified name
     * @param s the name
     */
    public JonasAdminTestCase(String s) {
        super(s);
        init();
    }

    /**
     * Constructor with a specified name and url
     * @param s the name
     * @param url the url which can be used
     */
    public JonasAdminTestCase(String s, String url) {
        super(s);
        wc = new WebConversation();
        init();
        this.url = getAbsoluteUrl(url);
    }
    /**
     * Constructor with a specified name and url
     * @param wc the WebConversation
     * @param s the name
     * @param url the url which can be used
     */
    public JonasAdminTestCase(WebConversation wc, String s,String url) {
        super(s);
        this.wc = wc;
        init();
        this.url = getAbsoluteUrl(url);
    }

    /**
     * Get initialContext
     * @return the initialContext
     * @throws NamingException if the initial context can't be retrieved
     */
    private Context getInitialContext() throws NamingException {
        return new InitialContext();
    }

    /**
     * Common setUp routine, used for every test.
     * @throws Exception if an error occurs
     */
    protected void setUp() throws Exception {
        try {
            // get InitialContext
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) PortableRemoteObject.narrow(ictx.lookup(jonasName + "_Adm"), AdmInterface.class);
            }

        } catch (NamingException e) {
            System.err.println("Cannot setup test: " + e);
            e.printStackTrace();
        }
    }


    /**
     * Load an ear file in the jonas server
     * @param filename ear file, without ".ear" extension
     * @throws Exception if an error occurs
     */
    public void useEar(String filename) throws Exception {

        try {
            // Load ear in JOnAS if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }

            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }

            //Test in both directories (apps/ and apps/autoload)
            String appsFileName = filename + ".ear";
            String autoloadAppsFileName = "autoload" + File.separator + filename + ".ear";
            if (!admI.isEarLoaded(appsFileName) && !admI.isEarLoaded(autoloadAppsFileName)) {
                //if the file was in autoload, it was loaded
                admI.addEar(appsFileName);
            }

        } catch (Exception e) {
            throw new Exception("Cannot load Ear : " + e.getMessage());
        }
    }


    /**
     * Unload a ear file in the jonas server
     * @param filename ear file, without ".ear" extension
     * @throws Exception if an error occurs
     */
    public void unUseEar(String filename) throws Exception {
        try {
            // Unload ear in EARServer if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            if (admI.isEarLoaded(filename + ".ear")) {
                admI.removeEar(filename + ".ear");
            }
        } catch (Exception e) {
            throw new Exception("Cannot unload Bean : " + e.getMessage());
        }
    }

    /**
     * Load a war file in the jonas server
     * @param filename war file, without ".war" extension
     * @throws Exception if an error occurs
     */
    public void useWar(String filename) throws Exception {

        try {
            // Load war in JOnAS if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }

            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }

            //Test in both directories (apps/ and apps/autoload)
            String webappsFileName = filename + ".war";
            String autoloadWebappsFileName = "autoload" + File.separator + filename + ".war";
            if (!admI.isWarLoaded(webappsFileName) && !admI.isWarLoaded(autoloadWebappsFileName)) {
                //if the file was in autoload, it was loaded
                admI.addWar(webappsFileName);
            }

        } catch (Exception e) {
            throw new Exception("Cannot load War : " + e.getMessage());
        }
    }

    /**
     * Load a bean jar file in the jonas server
     * @param filename jar file, without ".jar" extension
     * @throws Exception if an error occurs
     */
    public void useBeans(String filename) throws Exception {
        try {
            // Load bean in EJBServer if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            if (!admI.isLoaded(filename + ".jar")) {
                admI.addBeans(filename + ".jar");
            }
        } catch (Exception e) {
            throw new Exception("Cannot load Bean : " + e.getMessage());
        }
    }


    /**
     * Unload a bean jar file in the jonas server
     * @param filename jar file, without ".jar" extension
     * @throws Exception if an error occurs
     */
    public void unUseBeans(String filename) throws Exception {
        try {
            // Load bean in EJBServer if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            if (admI.isLoaded(filename + ".jar")) {
                admI.removeBeans(filename + ".jar");
            }
        } catch (Exception e) {
            throw new Exception("Cannot unload Bean : " + e.getMessage());
        }
    }


    /**
     * Load a rar file in the jonas server
     * @param filename rar file, without ".rar" extension
     * @throws Exception if an error occurs
     */
    public void useRar(String filename) throws Exception {

        try {
            // Load rar in JOnAS if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }

            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }

            //Test in both directories (rar/ and rar/autoload)
            String raFileName = filename + ".rar";
            String autoloadRaFileName = "autoload" + File.separator + filename + ".rar";
            if (!admI.isEarLoaded(raFileName) && !admI.isEarLoaded(autoloadRaFileName)) {
                //if the file was in autoload, it was loaded
                admI.addRar(raFileName);
            }

        } catch (Exception e) {
            throw new Exception("Cannot load Rar : " + e.getMessage());
        }
    }


    /**
     * Call the main method of a specific class with empty args
     * @param classToLoad name of class which contains the main method
     * @throws Exception if it fails
     */
    protected void callMainMethod(String classToLoad) throws Exception {
        callMainMethod(classToLoad, new String[]{});
    }


    /**
     * Call the main method of a specific class and the specific args
     * @param classToLoad name of class which contains the main method
     * @param args args to give to the main method
     * @throws Exception if it fails
     */
    protected void callMainMethod(String classToLoad, String[] args) throws Exception {
        //Build classloader
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        URL[] urls = new URL[1];
        urls[0] = new File(System.getProperty("jonas.root") + File.separator + "examples" + File.separator + "classes").toURL();
        URLClassLoader loader = new URLClassLoader(urls);
        Thread.currentThread().setContextClassLoader(loader);
        Class clazz = loader.loadClass(classToLoad);
        Class[] argList = new Class[] {args.getClass()};
        Method meth = clazz.getMethod("main", argList);
        Object appli = meth.invoke(null, new Object[]{args});
    }

    /**
     * Get current time
     * @return A String Time with the format yyyy-MM-dd.HH-mm-ss
     */
    protected String getTime() {
        Calendar cal = Calendar.getInstance();
        Date date = cal.getTime();
        SimpleDateFormat formatter;
        formatter = new SimpleDateFormat("yyyy-MM-dd.HH-mm-ss");
        return formatter.format(date);
    }

    /**
     * Undeploy an EAR
     * @param name String of EAR Name
     * @throws Exception if error occurs
     */
    protected void undeployEar(String name) throws Exception {
        String fileName = name + ".ear";

        // Disable errors of javascript
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);
        // Disable exception thrown on error status
        HttpUnitOptions.setExceptionsThrownOnErrorStatus(false);

        WebResponse wr = wc.getResponse(getAbsoluteUrl(URL_JONASADMIN_DEPLOYEAR));
        WebForm[] webForms = wr.getForms();
        WebForm webForm = webForms[0];

        String params = webForm.getParameterValue("undeploy");
        WebForm.Scriptable script = webForm.getScriptableObject();

        // Disable errors of javascript
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);

        if (params.length() == 0) {
            params += fileName;
        } else {
            params += "," + fileName;
        }

        script.setParameterValue("undeploy", params);

        WebResponse submitUndeploy = webForm.submit();

        webForms = submitUndeploy.getForms();
        webForm = webForms[0];

        WebResponse endResp = webForm.submit();
    }

    /**
     * Undeploy all EAR
     * @throws Exception if error occurs
     */
    protected void undeployAllEar() throws Exception {
        WebResponse wr = wc.getResponse(getAbsoluteUrl(URL_JONASADMIN_DEPLOYEAR));
        undeployAll(wr);
    }

    /**
     * Undeploy all Jars
     * @throws Exception if error occurs
     */
    protected void undeployAllJar() throws Exception {
        WebResponse wr = wc.getResponse(getAbsoluteUrl(URL_JONASADMIN_DEPLOYJAR));
        undeployAll(wr);
    }

    /**
     * Undeploy all
     * @throws Exception if error occurs
     */
    protected void undeployAll(WebResponse wr) throws Exception {

        WebForm[] webForms = wr.getForms();
        WebForm webForm = webForms[0];

        // Disable errors of javascript
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);
        // Disable exception thrown on error status
        HttpUnitOptions.setExceptionsThrownOnErrorStatus(false);

        String params = webForm.getParameterValue("deploy");
        WebForm.Scriptable script = webForm.getScriptableObject();

        if (params.length() == 0) {
            params += webForm.getParameterValue("undeploy");
        } else {
            params += "," + webForm.getParameterValue("undeploy");
        }

        script.setParameterValue("undeploy", params);

        WebResponse submitUndeploy = webForm.submit();

        webForms = submitUndeploy.getForms();
        webForm = webForms[0];

        WebResponse endResp = webForm.submit();

        wc.getResponse(getAbsoluteUrl(URL_JONASADMIN));
    }

    /**
     * Print a debug message
     * @param msg The message
     */
    public void debug(String msg) {
        System.out.println("DEBUG: " + msg);
    }
}
